/* Simple Normalize Filter

 * By Sylvain Deschenes
 *
 * Licensed under the terms of the GNU General Public License.
 * ABSOLUTELY NO WARRANTY.
 * See the file `COPYING' in this directory.
 */

#include "signpr_nor.h"
#include "signpr_general.h"
#include "errorwindow.h"
#include "stringinput.h"
#include "buttons.h"
#include "clrscr.h"
#include "boxes.h"
#include "helpline.h"
#include <stdlib.h>
#include <stdio.h>
#ifndef OLD_CURSES
#include <ncurses.h>
#else 
#include <curses.h>
#endif

int i;
long inpf;


void
simple_nor_param_defaults (parampointer_t parampointer)
{
  parampointer->postlength1 = 0;
  parampointer->prelength1 = 0;
}

void
simple_nor_param_screen (parampointer_t parampointer)
{
  stringinput_t norlengthstr;
  button_t ok_button, cancel_button;
  int dont_stop = TRUE;
  int returnval = 0;
  int focus = 0;
  int in_ch;

  long helplong;

  char *helplines[3] =
  {
    " ^: more expansion.                    v: less expansion                       ",
    " Discard changes.                                                              ",
    " Accept changes.                                                               "};

  norlengthstr.maxlen = 500;
  norlengthstr.string = (char *) malloc (norlengthstr.maxlen *
					  sizeof (char));
  sprintf (norlengthstr.string, "%ld", 0);
  norlengthstr.y = 6;
  norlengthstr.x = 38;
  norlengthstr.w = 15;
  norlengthstr.cursorpos = strlen (norlengthstr.string);
  norlengthstr.firstcharonscreen = 0;

  ok_button.text = " OK ";
  ok_button.y = 20;
  ok_button.x = 71;
  ok_button.selected = FALSE;

  cancel_button.text = " Cancel ";
  cancel_button.y = 20;
  cancel_button.x = 5;
  cancel_button.selected = FALSE;

  clearscreen (SIGNPR_NOR_PARAMSCR_HEADERTEXT);

  do
    {
      header (SIGNPR_NOR_PARAMSCR_HEADERTEXT);

      if (focus == 1)
	cancel_button.selected = TRUE;
      else
	cancel_button.selected = FALSE;

      if (focus == 2)
	ok_button.selected = TRUE;
      else
	ok_button.selected = FALSE;

      mvprintw (3, 2,
	    "Normalize filter - Increase or reduce signal by 0 to +/- 100 %%");

      mvprintw (4, 2,
	    "Use TRACK file to find maximum sample value and apply appropriate factor");

      stringinput_display (&norlengthstr);
      mvprintw (norlengthstr.y, 2,
		"Enter normalize factor:");

      button_display (&cancel_button);
      mybox (cancel_button.y - 1, cancel_button.x - 1,
	     3, strlen (cancel_button.text) + 2);
      button_display (&ok_button);
      mybox (ok_button.y - 1, ok_button.x - 1,
	     3, strlen (ok_button.text) + 2);

      helpline (helplines[focus]);

      if (focus == 0)
	stringinput_display (&norlengthstr);
      else
	move (0, 79);

      refresh ();

      in_ch = getch ();

      switch (focus)
	{
	case 0:		/* norlengthstr */
	  stringinput_stdkeys (in_ch, &norlengthstr);
	  if (in_ch == KEY_ENTER || in_ch == 13)
	    {
	      i = sscanf (norlengthstr.string, "%li", &helplong);

	      /*if (i < 0 || helplong < 0)
		error_window ("A number between 0 and 100 must \
be specified.");
	      else
               {*/
               inpf=helplong;
               focus = 2;
               /*} */
	    }
	  else
	    switch (in_ch)
	      {
	      case KEY_UP:
		focus--;
		break;
	      case KEY_DOWN:
		focus++;
		break;
	      }
	  break;

	case 1:		/* Cancel */
	  if (in_ch == KEY_ENTER || in_ch == 13)
	    {
	      returnval = 0;
	      dont_stop = FALSE;
	    }
	  else
	    switch (in_ch)
	      {
	      case KEY_LEFT:
	      case KEY_UP:
		focus--;
		break;
	      case KEY_RIGHT:
	      case KEY_DOWN:
		focus++;
		break;
	      }
	  break;

	case 2:		/* OK */
	  if (in_ch == KEY_ENTER || in_ch == 13)
	    {
	      i = sscanf (norlengthstr.string, "%li", &helplong);
	     /* if ( (abs) i < 0 || (abs) helplong < 0)
            {
		error_window ("A number between 0 and 100 must \
be specified.");

		  norlengthstr.cursorpos =
		    strlen (norlengthstr.string);
		  focus = 0;
		}
	      else
		{  */
                    inpf = helplong;
		    dont_stop = FALSE;
		/*}*/
	    }
	  else
	    switch (in_ch)
	      {
	      case KEY_LEFT:
	      case KEY_UP:
		focus--;
		break;
	      case KEY_RIGHT:
	      case KEY_DOWN:
		focus++;
		break;
	      }
	  break;
	}

      if (in_ch == 9)		/* TAB */
	focus++;

      if (in_ch == 27)
	dont_stop = FALSE;

      if (focus > 2)
	focus -= 3;
      if (focus < 0)
	focus += 3;
    }
  while (dont_stop);

  free (norlengthstr.string);
}

void
init_simple_nor_filter (int filterno, parampointer_t parampointer)
{
  parampointer->buffer = init_buffer (0,0);

  parampointer->filterno = filterno;
}

void
delete_simple_nor_filter (parampointer_t parampointer)
{
  delete_buffer (&parampointer->buffer);
}


sample_t
simple_nor_filter (parampointer_t parampointer)
{
 
  sample_t sample;
  double nfac;

  advance_current_pos (&parampointer->buffer, parampointer->filterno);


  sample = get_from_buffer (&parampointer->buffer,0);

  nfac = (double) 1 + (double) ((double) inpf / (double) 100);
  sample.left = (signed short)((double) sample.left * nfac);
  sample.right = (signed short)((double) sample.right * nfac);
 

  return sample;
}
